function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import React, { PureComponent } from "react";
import ReactDom from "react-dom";
import Pointable from "react-pointable";
import debounce from "lodash.debounce";
import { EventBus, PDFViewer, PDFLinkService } from "pdfjs-dist/web/pdf_viewer"; //$FlowFixMe

import "pdfjs-dist/web/pdf_viewer.css";
import "../style/pdf_viewer.css";
import "../style/PdfHighlighter.css";
import getBoundingRect from "../lib/get-bounding-rect";
import getClientRects from "../lib/get-client-rects";
import getAreaAsPng from "../lib/get-area-as-png";
import { asElement, getPageFromRange, getPageFromElement, getWindow, findOrCreateContainerLayer, isHTMLElement } from "../lib/pdfjs-dom";
import TipContainer from "./TipContainer";
import MouseSelection from "./MouseSelection";
import { scaledToViewport, viewportToScaled } from "../lib/coordinates";
import getClientRectsNew from '../lib/get-client-rects-new';
import getClientRectsNew1 from '../lib/get-client-rects-new1';
const EMPTY_ID = "empty-id";

class PdfHighlighter extends PureComponent {
  constructor(props) {
    super(props);

    _defineProperty(this, "state", {
      ghostHighlight: null,
      selection: null,
      isCollapsed: true,
      range: null,
      scrolledToHighlightId: EMPTY_ID,
      isAreaSelectionInProgress: false,
      tip: null,
      tipPosition: null,
      tipChildren: null
    });

    _defineProperty(this, "eventBus", new EventBus());

    _defineProperty(this, "linkService", new PDFLinkService({
      eventBus: this.eventBus,
      externalLinkTarget: 2
    }));

    _defineProperty(this, "viewer", void 0);

    _defineProperty(this, "resizeObserver", null);

    _defineProperty(this, "containerNode", null);

    _defineProperty(this, "unsubscribe", () => {});

    _defineProperty(this, "attachRef", ref => {
      const {
        eventBus,
        resizeObserver: observer
      } = this;
      this.containerNode = ref;
      this.unsubscribe();

      if (ref) {
        const {
          ownerDocument: doc
        } = ref;
        eventBus.on("textlayerrendered", this.onTextLayerRendered);
        eventBus.on("pagesinit", this.onDocumentReady);
        doc.addEventListener("click", this.onSelectionChange);
        doc.addEventListener("keydown", this.handleKeyDown);
        doc.defaultView.addEventListener("resize", this.debouncedScaleValue);
        if (observer) observer.observe(ref);

        this.unsubscribe = () => {
          eventBus.off("pagesinit", this.onDocumentReady);
          eventBus.off("textlayerrendered", this.onTextLayerRendered);
          doc.removeEventListener("click", this.onSelectionChange);
          doc.removeEventListener("keydown", this.handleKeyDown);
          doc.defaultView.removeEventListener("resize", this.debouncedScaleValue);
          if (observer) observer.disconnect();
        };
      }
    });

    _defineProperty(this, "scaledPositionToViewport", ({
      pageNumber,
      boundingRect,
      rects,
      usePdfCoordinates
    }) => {
      const viewport = this.viewer.getPageView(pageNumber - 1).viewport;
      return {
        boundingRect: scaledToViewport(boundingRect, viewport, usePdfCoordinates),
        rects: (rects || []).map(rect => scaledToViewport(rect, viewport, usePdfCoordinates)),
        pageNumber
      };
    });

    _defineProperty(this, "viewportRectsToScaled", (rects, pageNumber) => {
      const viewport = this.viewer.getPageView(pageNumber).viewport;
      return (rects || []).map(i => {
        const points = [i.points[0], i.points[1], i.points[0] + i.points[2], i.points[1] + i.points[3]];
        const bounds = viewport.convertToViewportRectangle(points);
        return {
          left: Math.min(bounds[0], bounds[2]),
          top: Math.min(bounds[1], bounds[3]) - (i.isUnderline ? 0 : 2.5),
          width: Math.abs(bounds[0] - bounds[2]),
          height: Math.abs(bounds[1] - bounds[3]) + (i.isUnderline ? 0 : 4.5),
          page: i.page
        };
      }); // return (rects || []).map(rect => scaledToViewport(rect, viewport));
    });

    _defineProperty(this, "clearSelection", (range, startPage) => {
      // this.setState({ selection: null }, this.renderSelections);
      this.renderSelections(this.props, range, startPage);
    });

    _defineProperty(this, "hideTipAndSelection", () => {
      this.setState({
        tipPosition: null,
        tipChildren: null
      });
      this.setState({
        ghostHighlight: null,
        tip: null
      }, () => this.renderHighlights());
    });

    _defineProperty(this, "onTextLayerRendered", () => {
      this.renderNotes();
      this.renderDefinitions();
      this.renderHighlights();
      this.renderPreSelections();
    });

    _defineProperty(this, "scrollTo", highlight => {
      const {
        pages,
        rects
      } = highlight;
      const pageNumber = pages[0];
      this.viewer.container.removeEventListener("scroll", this.onScroll);
      this.viewer.scrollPageIntoView({
        pageNumber,
        destArray: [null, {
          name: "XYZ"
        }, rects[0].points[0], rects[0].points[1], 0]
      });
      this.setState({
        scrolledToHighlightId: highlight.id
      }, () => this.renderHighlights()); // wait for scrolling to finish

      setTimeout(() => {
        this.viewer.container.addEventListener("scroll", this.onScroll);
      }, 100);
    });

    _defineProperty(this, "onDocumentReady", () => {
      const {
        scrollRef
      } = this.props;
      this.handleScaleValue();
      scrollRef(this.scrollTo);
    });

    _defineProperty(this, "onSelectionChange", () => {
      const container = this.containerNode;
      const selection = getWindow(container).getSelection();
      const range = selection.rangeCount > 0 ? selection.getRangeAt(0) : null;

      if (selection && selection.baseNode && selection.baseNode.parentNode && (selection.baseNode.parentNode.className.includes("definition") || selection.baseNode.parentNode.className.includes("body") || selection.baseNode.parentNode.className.includes("body-title") || selection.baseNode.parentNode.className.includes("body-description") || selection.baseNode.parentNode.className.includes("popover"))) {
        return null;
      }

      if (selection.isCollapsed) {
        this.setState({
          isCollapsed: true
        });
        return;
      }

      if (!range || !container || !container.contains(range.commonAncestorContainer)) {
        return;
      }

      this.setState({
        isCollapsed: false,
        range
      });
      this.afterSelection(); // this.debouncedAfterSelection();
    });

    _defineProperty(this, "onScroll", () => {
      const {
        onScrollChange
      } = this.props;
      onScrollChange();
      this.setState({
        scrolledToHighlightId: EMPTY_ID
      }, () => this.renderHighlights());
      this.viewer.container.removeEventListener("scroll", this.onScroll);
    });

    _defineProperty(this, "onMouseDown", event => {
      if (!isHTMLElement(event.target)) {
        return;
      }

      if (asElement(event.target).closest(".PdfHighlighter__tip-container")) {
        return;
      }

      this.hideTipAndSelection();
    });

    _defineProperty(this, "handleKeyDown", event => {
      if (event.code === "Escape") {
        this.hideTipAndSelection();
      }
    });

    _defineProperty(this, "afterSelection", () => {
      const {
        onSelectionFinished,
        withoutSelection
      } = this.props;
      const {
        isCollapsed,
        range
      } = this.state;

      if (!range || isCollapsed) {
        return;
      }

      const [startPage, endPage] = getPageFromRange(range);

      if (!startPage || !endPage) {
        return;
      }

      const rects = getClientRectsNew(range, [startPage, endPage], this);
      const rectsPdf = getClientRectsNew1(range, [startPage, endPage], this);
      const scaledRects = this.viewportSelectionRectsToScaled(rects); //
      // const p1_1 = this.viewer.getPageView(rects[0].page - 1).viewport.convertToPdfPoint(rects[0].top, rects[0].left);
      // const p1_2 = this.viewer.getPageView(rects[0].page - 1).viewport.convertToPdfPoint(rects[0].top + rects[0].height, rects[0].left + rects[0].width);
      // const p2_1 = this.viewer.getPageView(rects[0].page - 1).viewport.convertToPdfPoint(rects[1].top, rects[1].left);
      // const p2_2 = this.viewer.getPageView(rects[0].page - 1).viewport.convertToPdfPoint(rects[0].top + rects[0].height, rects[1].left + rects[1].width);
      // const w1 = this.viewer.getPageView(rects[0].page - 1).viewport.convertToPdfPoint(p1_2[0] - p1_1[0], 0);
      // const h1 = this.viewer.getPageView(rects[0].page - 1).viewport.convertToPdfPoint(p1_2[1] - p1_1[1], 0);
      // const w2 = this.viewer.getPageView(rects[0].page - 1).viewport.convertToPdfPoint(p2_2[0] - p2_1[0], 0);
      // const h2 = this.viewer.getPageView(rects[0].page - 1).viewport.convertToPdfPoint(p2_2[1] - p2_1[1], 0);
      // const rectsPdf = [
      //   {
      //     x1: +p1_1[0].toFixed(2),
      //     y1: +p1_1[1].toFixed(2),
      //     x2: +p1_2[0].toFixed(2),
      //     y2: +p1_2[1].toFixed(2),
      //     w: +w1[1].toFixed(2),
      //     h: +h1[1].toFixed(2),
      //     page: rects[0].page
      //   },
      //   {
      //     x1: +p2_1[0].toFixed(2),
      //     y1: +p2_1[1].toFixed(2),
      //     x2: +p2_2[0].toFixed(2),
      //     y2: +p2_2[1].toFixed(2),
      //     w: +w2[1].toFixed(2),
      //     h: +h2[1].toFixed(2),
      //     page: rects[1].page
      //   }
      // ];
      // const rects = getClientRects(range, fromPage, true);
      // const rects = getClientRectsNew(range, page, true, this);
      //
      // if (rects.length === 0) {
      //   return;
      // }
      //
      // const boundingRect = getBoundingRect(rects);
      // const viewportPosition = { boundingRect, rects, pageNumber: page.number };
      //
      // console.log("test, ", "afterSelection --- ", boundingRect, viewportPosition);
      //
      // const content = {
      //   text: range.toString()
      // };
      // const scaledPosition = this.viewportPositionToScaled(viewportPosition);
      //
      // if (!withoutSelection) {
      //   this.setState({ selection: { position: scaledPosition } }, this.renderSelections);
      //   window.getSelection().removeAllRanges();
      // }

      onSelectionFinished({
        range,
        startPage,
        endPage,
        rects: scaledRects,
        rectsPdf
      });
      this.renderSelections(this.props, range, startPage.number, scaledRects);
    });

    _defineProperty(this, "debouncedAfterSelection", debounce(this.afterSelection, 500));

    _defineProperty(this, "handleScaleValue", () => {
      if (this.viewer) {
        this.viewer.currentScaleValue = this.props.pdfScaleValue; //"page-width";
      }
    });

    _defineProperty(this, "debouncedScaleValue", debounce(this.handleScaleValue, 500));

    if (typeof ResizeObserver !== "undefined") {
      this.resizeObserver = new ResizeObserver(this.debouncedScaleValue);
    }
  }

  componentDidMount() {
    console.log("PdfHighlighter, ", "componentDidMount --- ");
    this.init();
  }

  componentDidUpdate(prevProps) {
    if (prevProps.pdfDocument !== this.props.pdfDocument) {
      this.init();
      return;
    }

    if (prevProps.highlights !== this.props.highlights) {
      this.renderHighlights(this.props);
    }

    if (prevProps.definitions !== this.props.definitions) {
      this.renderDefinitions(this.props);
    }

    if (prevProps.notes !== this.props.notes) {
      this.renderNotes(this.props);
    }

    if (prevProps.preSelections !== this.props.preSelections) {
      this.renderPreSelections(this.props);
    }

    if (prevProps.pdfScaleValue !== this.props.pdfScaleValue) {
      this.handleScaleValue();
    }
  }

  init() {
    const {
      pdfDocument
    } = this.props;
    this.viewer = this.viewer || new PDFViewer({
      container: this.containerNode,
      eventBus: this.eventBus,
      enhanceTextSelection: true,
      removePageBorders: true,
      linkService: this.linkService,
      textLayerMode: 2
    });
    this.linkService.setDocument(pdfDocument);
    this.linkService.setViewer(this.viewer);
    this.viewer.setDocument(pdfDocument);
    this.props.onInit(this); // debug

    window.PdfViewer = this;
  }

  componentWillUnmount() {
    this.unsubscribe();
  }

  findOrCreateHighlightLayer(page) {
    const {
      textLayer
    } = this.viewer.getPageView(page - 1) || {};

    if (!textLayer) {
      return null;
    }

    return findOrCreateContainerLayer(textLayer.textLayerDiv, "PdfHighlighter__highlight-layer");
  }

  findOrCreateLabelLayer(page) {
    const {
      textLayer
    } = this.viewer.getPageView(page - 1) || {};

    if (!textLayer) {
      return null;
    }

    return findOrCreateContainerLayer(textLayer.textLayerDiv.parentNode, "PdfHighlighter__labels-layer");
  }

  findOrCreateDefinitionLayer(page) {
    const {
      textLayer
    } = this.viewer.getPageView(page - 1) || {};

    if (!textLayer) {
      return null;
    }

    return findOrCreateContainerLayer(textLayer.textLayerDiv.parentNode, "PdfHighlighter__definitions-layer");
  }

  findOrCreateNotesLayer(page) {
    const {
      textLayer
    } = this.viewer.getPageView(page - 1) || {};

    if (!textLayer) {
      return null;
    }

    return findOrCreateContainerLayer(textLayer.textLayerDiv.parentNode, "PdfHighlighter__notes-layer");
  }

  findOrCreateSelectionLayer(page) {
    const {
      textLayer
    } = this.viewer.getPageView(page - 1) || {};

    if (!textLayer) {
      return null;
    }

    return findOrCreateContainerLayer(textLayer.textLayerDiv.parentNode, "PdfHighlighter__selections-layer");
  }

  findOrCreatePreSelectionLayer(page) {
    const {
      textLayer
    } = this.viewer.getPageView(page - 1) || {};

    if (!textLayer) {
      return null;
    }

    return findOrCreateContainerLayer(textLayer.textLayerDiv.parentNode, "PdfHighlighter__pre-selections-layer");
  }

  groupHighlightsByPage(highlights) {
    return highlights.filter(Boolean).reduce((res, highlight) => {
      highlight.pages.forEach(page => {
        res[page] = res[page] || [];
        res[page].push(highlight);
      });
      return res;
    }, {});
  }

  viewportPositionToScaled({
    pageNumber,
    boundingRect,
    rects
  }) {
    const viewport = this.viewer.getPageView(pageNumber - 1).viewport;
    return {
      boundingRect: viewportToScaled(boundingRect, viewport),
      rects: (rects || []).map(rect => viewportToScaled(rect, viewport)),
      pageNumber
    };
  }

  viewportSelectionRectsToScaled(rects) {
    return [viewportToScaled(rects[0], this.viewer.getPageView(rects[0].page - 1).viewport), viewportToScaled(rects[1], this.viewer.getPageView(rects[1].page - 1).viewport)];
  } // screenshot(position: T_LTWH, pageNumber: number) {
  //   const canvas = this.viewer.getPageView(pageNumber - 1).canvas;
  //
  //   return getAreaAsPng(canvas, position);
  // }


  renderHighlights(nextProps) {
    const {
      highlightTransform,
      labelTransform,
      highlights
    } = nextProps || this.props;
    const {
      pdfDocument
    } = this.props;
    const {
      scrolledToHighlightId
    } = this.state;
    const highlightsByPage = this.groupHighlightsByPage(highlights);

    for (let pageNumber = 1; pageNumber <= pdfDocument.numPages; pageNumber++) {
      const highlightLayer = this.findOrCreateHighlightLayer(pageNumber);
      const labelLayer = this.findOrCreateLabelLayer(pageNumber);

      if (highlightLayer) {
        ReactDom.render( /*#__PURE__*/React.createElement("div", null, (highlightsByPage[String(pageNumber)] || []).map(({
          rects,
          id,
          ...highlight
        }, index) => {
          const viewportHighlight = {
            id,
            rects: this.viewportRectsToScaled(rects, pageNumber - 1),
            actualPage: pageNumber - 1,
            ...highlight
          };
          const isScrolledTo = Boolean(scrolledToHighlightId === id);
          return highlightTransform(viewportHighlight, index, isScrolledTo);
        })), highlightLayer);
      }

      if (labelLayer) {
        ReactDom.render( /*#__PURE__*/React.createElement("div", null, (highlightsByPage[String(pageNumber)] || []).map(({
          rects,
          id,
          ...highlight
        }, index) => {
          const viewportHighlight = {
            id,
            rects: this.viewportRectsToScaled(rects, pageNumber - 1),
            actualPage: pageNumber - 1,
            ...highlight
          };
          return labelTransform(viewportHighlight, index);
        })), labelLayer);
      }
    }
  }

  renderDefinitions(nextProps) {
    const {
      definitionTransform,
      definitions
    } = nextProps || this.props;
    const {
      pdfDocument
    } = this.props;
    const definitionsByPage = this.groupHighlightsByPage(definitions);

    for (let pageNumber = 1; pageNumber <= pdfDocument.numPages; pageNumber++) {
      const definitionLayer = this.findOrCreateDefinitionLayer(pageNumber);

      if (definitionLayer) {
        ReactDom.render( /*#__PURE__*/React.createElement("div", null, (definitionsByPage[String(pageNumber)] || []).map(({
          rects,
          id,
          ...highlight
        }, index) => {
          const viewportHighlight = {
            id,
            rects: this.viewportRectsToScaled(rects, pageNumber - 1),
            actualPage: pageNumber - 1,
            ...highlight
          };
          return definitionTransform(viewportHighlight, index);
        })), definitionLayer);
      }
    }
  }

  renderNotes(nextProps) {
    const {
      noteTransform,
      notes
    } = nextProps || this.props;
    const {
      pdfDocument
    } = this.props;
    const notesByPage = this.groupHighlightsByPage(notes);

    for (let pageNumber = 1; pageNumber <= pdfDocument.numPages; pageNumber++) {
      const notesLayer = this.findOrCreateNotesLayer(pageNumber);

      if (notesLayer) {
        ReactDom.render( /*#__PURE__*/React.createElement("div", null, (notesByPage[String(pageNumber)] || []).map(({
          rects,
          id,
          ...highlight
        }, index) => {
          const viewportHighlight = {
            id,
            rects: this.viewportRectsToScaled(rects, pageNumber - 1),
            actualPage: pageNumber - 1,
            ...highlight
          };
          return noteTransform(viewportHighlight, index);
        })), notesLayer);
      }
    }
  }

  renderSelections(nextProps, range, page, rects) {
    const {
      selectionTransform
    } = nextProps || this.props;
    const selectionLayer = this.findOrCreateSelectionLayer(page);

    if (selectionLayer) {
      const sTransform = range ? selectionTransform({
        range,
        page,
        rects
      }) : [];
      ReactDom.render( /*#__PURE__*/React.createElement("div", null, sTransform), selectionLayer);
    }
  }

  renderPreSelections(nextProps) {
    const {
      preSelectionTransform,
      preSelections
    } = nextProps || this.props;
    const {
      pdfDocument
    } = this.props;
    const preSelectionsByPage = this.groupHighlightsByPage(preSelections);

    for (let pageNumber = 1; pageNumber <= pdfDocument.numPages; pageNumber++) {
      const preSelectionLayer = this.findOrCreatePreSelectionLayer(pageNumber);

      if (preSelectionLayer) {
        ReactDom.render( /*#__PURE__*/React.createElement("div", null, (preSelectionsByPage[String(pageNumber)] || []).map(({
          rects,
          id,
          ...highlight
        }, index) => {
          const viewportHighlight = {
            id,
            rects: this.viewportRectsToScaled(rects, pageNumber - 1),
            actualPage: pageNumber - 1,
            ...highlight
          };
          return preSelectionTransform(viewportHighlight, index);
        })), preSelectionLayer);
      }
    }
  }

  toggleTextSelection(flag) {
    this.viewer.viewer.classList.toggle("PdfHighlighter--disable-selection", flag);
  }

  render() {
    return /*#__PURE__*/React.createElement(Pointable, {
      onPointerDown: this.onMouseDown
    }, /*#__PURE__*/React.createElement("div", {
      ref: this.attachRef,
      className: "PdfHighlighter",
      onContextMenu: e => e.preventDefault()
    }, /*#__PURE__*/React.createElement("div", {
      className: "pdfViewer"
    })));
  }

}

_defineProperty(PdfHighlighter, "defaultProps", {
  pdfScaleValue: "auto",
  withoutSelection: false,
  labelTransform: () => null,
  definitionTransform: () => null,
  noteTransform: () => null,
  selectionTransform: () => null,
  onInit: () => null,
  definitions: [],
  notes: []
});

export default PdfHighlighter;